@@ -21,8 +21,8 @@
 #include <qcommandlineparser.h>
 #include <qobject.h>
 #include <qtmetamacros.h>
-#include <KLocalizedString>
 #include <KAboutData>
+#include <KLocalizedString>
 #include <QApplication>
 #include <QLoggingCategory>
 #include <QString>
@@ -34,14 +34,14 @@
 #include "pipeline_type.hpp"
 #include "presets_manager.hpp"
 
-CommandLineParser::CommandLineParser(KAboutData &about, QObject* parent)
+CommandLineParser::CommandLineParser(KAboutData& about, QObject* parent)
     : QObject(parent), parser(std::make_unique<QCommandLineParser>()) {
   parser->setApplicationDescription("Easy Effects");
 
   about.setupCommandLine(parser.get());
   parser->addOptions({{{"q", "quit"}, i18n("Quit Easy Effects. Useful when running in service mode.")},
                       {{"r", "reset"}, i18n("Reset Easy Effects.")},
-                      {{"w", "hide-window"}, i18n("Hide the Window.")},
+                      {{"w", "hide-window"}, i18n("Hide the window.")},
                       {{"b", "bypass"}, i18n("Global bypass. 1 to enable and 2 to disable."), i18n("bypass-state")},
                       {{"l", "load-preset"}, i18n("Load a preset. Example: easyeffects -l music"), i18n("preset-name")},
                       {{"p", "presets"}, i18n("Show available presets.")},
@@ -52,12 +52,30 @@ CommandLineParser::CommandLineParser(KAboutData &about, QObject* parent)
                       {"debug", i18n("Enable debug messages.")}});
 }
 
-void CommandLineParser::process(KAboutData &about, QApplication* app) {
-  auto* pm = &presets::Manager::self();
+void CommandLineParser::set_is_primary(const bool& state) {
+  is_primary = state;
+}
 
+void CommandLineParser::process(KAboutData& about, QApplication* app) {
   parser->process(*app);
 
   about.processCommandLine(parser.get());
+}
+
+void CommandLineParser::process_debug_option() {
+  if (parser->isSet("debug")) {
+    QLoggingCategory::setFilterRules("easyeffects.debug=true");
+  }
+}
+
+void CommandLineParser::process_hide_window(bool& show_window) {
+  if (parser->isSet("hide-window")) {
+    show_window = false;
+  }
+}
+
+void CommandLineParser::process_events() {
+  auto* pm = &presets::Manager::self();
 
   if (parser->isSet("quit")) {
     Q_EMIT onQuit();
@@ -81,10 +99,6 @@ void CommandLineParser::process(KAboutData &about, QApplication* app) {
     Q_EMIT onHideWindow();
   }
 
-  if (parser->isSet("debug")) {
-    QLoggingCategory::setFilterRules("easyeffects.debug=true");
-  }
-
   if (parser->isSet("active-preset")) {
     bool ok = true;
 
@@ -142,20 +156,30 @@ void CommandLineParser::process(KAboutData &about, QApplication* app) {
 
   if (parser->isSet("presets")) {
     std::string list;
+    int i = 0;
 
     for (const auto& p : pm->get_local_presets_paths(PipelineType::output)) {
-      list += p.stem().string() + ",";
+      list += util::to_string(++i) + '\t' + p.stem().string() + '\n';
     }
 
-    std::cout << i18n("Output Presets").toStdString() + ": " + list << '\n';
+    if (i > 0) {
+      std::cout << i18n("Output presets").toStdString() + ":\n" + list << '\n';
+    } else {
+      std::cout << i18n("No output presets.").toStdString() << '\n';
+    }
 
     list = "";
+    i = 0;
 
     for (const auto& p : pm->get_local_presets_paths(PipelineType::input)) {
-      list += p.stem().string() + ",";
+      list += util::to_string(++i) + '\t' + p.stem().string() + '\n';
     }
 
-    std::cout << i18n("Input Presets").toStdString() + ": " + list << '\n';
+    if (i > 0) {
+      std::cout << i18n("Input presets").toStdString() + ":\n" + list << '\n';
+    } else {
+      std::cout << i18n("No input presets.").toStdString() << '\n';
+    }
 
     Q_EMIT onHideWindow();
 
@@ -163,23 +187,31 @@ void CommandLineParser::process(KAboutData &about, QApplication* app) {
   }
 
   if (parser->isSet("load-preset")) {
+    bool ok = false;
+
     const auto name = parser->value("load-preset");
 
     if (pm->preset_file_exists(PipelineType::input, name.toStdString())) {
       Q_EMIT onLoadPreset(PipelineType::input, name);
 
-      QCoreApplication::exit(EXIT_SUCCESS);
+      ok = true;
     }
 
     if (pm->preset_file_exists(PipelineType::output, name.toStdString())) {
       Q_EMIT onLoadPreset(PipelineType::output, name);
 
-      QCoreApplication::exit(EXIT_SUCCESS);
+      ok = true;
     }
 
     Q_EMIT onHideWindow();
 
-    QCoreApplication::exit(EXIT_FAILURE);
+    if (ok) {
+      QCoreApplication::exit(EXIT_SUCCESS);
+    } else {
+      std::cout << i18n("Specified preset does not exist.").toStdString() << '\n';
+
+      QCoreApplication::exit(EXIT_FAILURE);
+    }
   }
 
   if (parser->isSet("bypass")) {
@@ -191,12 +223,22 @@ void CommandLineParser::process(KAboutData &about, QApplication* app) {
       Q_EMIT onSetGlobalBypass(true);
     } else if (value == "2") {
       Q_EMIT onSetGlobalBypass(false);
+    } else {
+      ok = false;
+
+      std::cout << i18n("Provided an invalid bypass state.").toStdString() << '\n';
     }
 
+    Q_EMIT onHideWindow();
+
     if (ok) {
       QCoreApplication::exit(EXIT_SUCCESS);
     } else {
       QCoreApplication::exit(EXIT_FAILURE);
     }
   }
+
+  if (is_primary) {
+    Q_EMIT onInitQML();
+  }
 }
