@@ -20,22 +20,39 @@
 #pragma once
 
 #include <qlist.h>
+#include <qobject.h>
 #include <qpoint.h>
 #include <qtmetamacros.h>
 #include <sys/types.h>
 #include <zita-convolver.h>
 #include <QString>
-#include <format>
+#include <QThread>
 #include <span>
 #include <string>
-#include <thread>
 #include <vector>
+#include "convolver_kernel_fft.hpp"
 #include "convolver_kernel_manager.hpp"
+#include "convolver_zita.hpp"
 #include "easyeffects_db_convolver.h"
 #include "pipeline_type.hpp"
 #include "plugin_base.hpp"
 #include "pw_manager.hpp"
-#include "util.hpp"
+
+class ConvolverWorker : public QObject {
+  Q_OBJECT
+
+ public Q_SLOTS:
+  void calculateFFT(std::vector<float> kernel_L, std::vector<float> kernel_R, int kernel_rate, int interpPoints);
+
+ Q_SIGNALS:
+  void onNewKernel(ConvolverKernelManager::KernelData data);
+
+  void onNewChartMag(QList<QPointF> mag_L, QList<QPointF> mag_R);
+
+  void onNewSpectrum(QList<QPointF> linear_L, QList<QPointF> linear_R, QList<QPointF> log_L, QList<QPointF> log_R);
+
+  void onInvalidKernel(QString name);
+};
 
 class Convolver : public PluginBase {
   Q_OBJECT
@@ -82,8 +99,6 @@ class Convolver : public PluginBase {
 
   auto get_latency_seconds() -> float override;
 
-  const std::string irs_ext = ".irs";
-
   Q_INVOKABLE void combineKernels(const QString& kernel1, const QString& kernel2, const QString& outputName);
 
  Q_SIGNALS:
@@ -111,8 +126,8 @@ class Convolver : public PluginBase {
 
   bool kernel_is_initialized = false;
   bool n_samples_is_power_of_2 = true;
-  bool zita_ready = false;
   bool ready = false;
+  bool destructor_called = false;
   bool notify_latency = false;
 
   uint blocksize = 512U;
@@ -127,7 +142,6 @@ class Convolver : public PluginBase {
   QString kernelDuration;
 
   std::vector<float> data_L, data_R;
-
   std::vector<float> buf_in_L, buf_in_R;
   std::vector<float> buf_out_L, buf_out_R;
 
@@ -135,60 +149,23 @@ class Convolver : public PluginBase {
 
   ConvolverKernelManager kernel_manager;
 
-  ConvolverKernelManager::KernelData kernel, original_kernel;
+  ConvolverKernelFFT kernel_fft;
 
-  Convproc* conv = nullptr;
+  ConvolverZita zita;
 
-  std::vector<std::thread> mythreads;
+  ConvolverWorker* worker;
 
-  void apply_kernel_autogain();
+  QThread workerThread;
 
-  void set_kernel_stereo_width();
-
-  void setup_zita();
-
-  auto get_zita_buffer_size() -> uint;
-
-  void prepare_kernel();
-
-  void load_kernel_file();
+  void load_kernel_file(const bool& init_zita, const uint& server_sampling_rate);
 
   void combine_kernels(const std::string& kernel_1_name,
                        const std::string& kernel_2_name,
                        const std::string& output_file_name);
 
-  static auto interpolate(const std::vector<double>& x_source,
-                          const std::vector<double>& y_source,
-                          const std::vector<double>& x_new) -> std::vector<double>;
-
   void chart_kernel_fft(const std::vector<float>& kernel_L,
                         const std::vector<float>& kernel_R,
                         const float& kernel_rate);
 
   void clear_chart_data();
-
-  template <typename T1>
-  void do_convolution(T1& data_left, T1& data_right) {
-    std::span conv_left_in(conv->inpdata(0), get_zita_buffer_size());
-    std::span conv_right_in(conv->inpdata(1), get_zita_buffer_size());
-
-    std::span conv_left_out(conv->outdata(0), get_zita_buffer_size());
-    std::span conv_right_out(conv->outdata(1), get_zita_buffer_size());
-
-    std::copy(data_left.begin(), data_left.end(), conv_left_in.begin());
-    std::copy(data_right.begin(), data_right.end(), conv_right_in.begin());
-
-    if (zita_ready) {
-      const int& ret = conv->process(true);  // thread sync mode set to true
-
-      if (ret != 0) {
-        util::debug(std::format("{}IR: process failed: {}", log_tag, ret));
-
-        zita_ready = false;
-      } else {
-        std::copy(conv_left_out.begin(), conv_left_out.end(), data_left.begin());
-        std::copy(conv_right_out.begin(), conv_right_out.end(), data_right.begin());
-      }
-    }
-  }
 };
